#VERTEX_CODE
precision highp float;

attribute vec4 vert_position;
attribute vec4 text_coord;
varying   mediump vec2 output_text_coord;

void main() {
    gl_Position       = vert_position;
    output_text_coord = text_coord.xy;
}
#END_CODE

#FRAGMENT_CODE
precision mediump float;

varying vec2  output_text_coord;
uniform float smooth_intensity;
uniform float variance_tolerance;
uniform int   face_num;

uniform sampler2D input_img;
uniform sampler2D blur_img;
uniform sampler2D var_skin_mask_img;
uniform sampler2D face_mask_img;
uniform sampler2D highpass_ori_img;
uniform sampler2D highpass_blur_img;

uniform bool  use_highpass;
uniform bool  use_bangs_clarity;

void main()
{
    lowp vec4 orig_color = texture2D(input_img, output_text_coord).rgba;
    lowp vec3 mean_color = texture2D(blur_img,  output_text_coord).rgb;
    float variance = texture2D(var_skin_mask_img, output_text_coord).r;  // range [0, 1]

    // skin probability and darkness attenuation
    mediump float orig_skin_prob = texture2D(var_skin_mask_img, output_text_coord).g;
    mediump float skin_prob      = orig_skin_prob;
    mediump vec4  face_mask      = vec4(0.0, 0.0, 0.0, 0.0);
    if (face_num > 0)
    {
        face_mask = texture2D(face_mask_img, output_text_coord).rgba;
        skin_prob = face_mask.b > 0.005 ? min(orig_skin_prob, face_mask.r) : 0.0;
        skin_prob = use_bangs_clarity ? skin_prob * (1.0 - face_mask.a) : skin_prob; // for bangs clarity

        if (use_highpass)
        {
            vec3 ori_highpass = texture2D(highpass_ori_img, output_text_coord).rgb;
            vec3 blur_highpass = texture2D(highpass_blur_img, output_text_coord).rgb;
            float blemish_prob = (ori_highpass.r + ori_highpass.g + ori_highpass.b > 1.5) ? 1.0 : 0.2;
            vec3 negative_blemish = mix(blur_highpass, ori_highpass, blemish_prob);
            mean_color = clamp(orig_color.rgb + 2.0 * negative_blemish - 1.0, 0.0, 1.0);
        }
    }

    // variance attenuation
    mediump float variance_attenuation = variance_tolerance / (variance + variance_tolerance);

    // blend
    mediump float final_blurred_ratio = clamp(skin_prob * variance_attenuation * smooth_intensity, 0.0, 1.0);
    mediump vec3  result_color        = mix(orig_color.rgb, mean_color, final_blurred_ratio);

    gl_FragColor = vec4(result_color, orig_color.a);
}
#END_CODE