#VERTEX_CODE
precision highp float;

attribute vec4 vert_position;
attribute vec4 text_coord;

uniform float pixel_width_offset;
uniform float pixel_height_offset;

varying mediump vec2 output_text_coord;
varying mediump vec4 pixel_shift_1;
varying mediump vec4 pixel_shift_2;

void main()
{
    gl_Position = vert_position;
    output_text_coord = text_coord.xy;

    vec2 pixel_step_offset = vec2(pixel_width_offset, pixel_height_offset);

    pixel_shift_1 = vec4(text_coord.xy - 1.0 * pixel_step_offset, text_coord.xy + 1.0 * pixel_step_offset);
    pixel_shift_2 = vec4(text_coord.xy - 2.0 * pixel_step_offset, text_coord.xy + 2.0 * pixel_step_offset);
}
#END_CODE

#FRAGMENT_CODE
precision mediump float;

uniform sampler2D input_image;
uniform sampler2D facemask_image;
uniform float distance_magnification;

varying mediump vec2 output_text_coord;
varying mediump vec4 pixel_shift_1;
varying mediump vec4 pixel_shift_2;

void main()
{
    float facemask_b =  texture2D(facemask_image, output_text_coord).b;
    vec3 center_color = texture2D(input_image, output_text_coord).rgb;
    vec3 result = center_color;

    if (facemask_b > 0.001)
    {
        mediump vec3  img_color;
        mediump vec3  color_diff;
        mediump vec3  sum;
        mediump float pixel_wt;
        mediump float wt;

        sum = center_color;
        wt  = 1.0;

        img_color  =  texture2D(input_image, pixel_shift_1.xy).rgb;
        color_diff =  abs(img_color - center_color);
        pixel_wt   =  max(1.0 - (color_diff.r + color_diff.g + color_diff.b) * distance_magnification, 0.0);
        sum        += img_color * pixel_wt;
        wt         += pixel_wt;

        img_color  =  texture2D(input_image, pixel_shift_1.zw).rgb;
        color_diff =  abs(img_color - center_color);
        pixel_wt   =  max(1.0 - (color_diff.r + color_diff.g + color_diff.b) * distance_magnification, 0.0);
        sum        += img_color * pixel_wt;
        wt         += pixel_wt;

        img_color  =  texture2D(input_image, pixel_shift_2.xy).rgb;
        color_diff =  abs(img_color - center_color);
        pixel_wt   =  max(1.0 - (color_diff.r + color_diff.g + color_diff.b) * distance_magnification, 0.0);
        sum        += img_color * pixel_wt;
        wt         += pixel_wt;

        img_color  =  texture2D(input_image, pixel_shift_2.zw).rgb;
        color_diff =  abs(img_color - center_color);
        pixel_wt   =  max(1.0 - (color_diff.r + color_diff.g + color_diff.b) * distance_magnification, 0.0);
        sum        += img_color * pixel_wt;
        wt         += pixel_wt;

        result = sum / wt;
    }

    gl_FragColor = vec4(result, 1.0);
}
#END_CODE